package org.ow2.choreos.gui.visualComponentConnection;


import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;


public class VisualComponentConnection extends JPanel {

    /**
	 * 
	 */
	private static final long serialVersionUID = -2722426160622180233L;

	/**An instance of {@code ConnectLine} indicating the base line of this {@code VisualComponentConnection}*/
    protected ConnectLine line;

    /**An instance of {@link VisualComponentConnectionVisibilityProperties VisualComponentConnectionVisibilityProperties}
     * indicating the visibility properties of this {@code VisualComponentConnection}
     */
    protected VisualComponentConnectionVisibilityProperties visibilityProp;

    /**An instance of {@link VisualComponentConnectionLineProperties VisualComponentConnectionLineProperties}
     * indicating the line properties of this {@code VisualComponentConnection}
     */
    protected VisualComponentConnectionLineProperties lineProp;
    /**A {@code JComponent} indicating the original source of this {@code VisualComponentConnection}.
     *This {@code JComponent}'s bounds are changed when the source TreeNode of this {@code VisualComponentConnection}
     *is a descendant of a TreeNode that get's expanded or collapsed
     *@see VisualComponentConnectionUtils#moveBelowConnectionsBy
     */
    protected JComponent originalSource;

    /**A {@code JComponent} indicating the original destination of this {@code VisualComponentConnection}.
     *This {@code JComponent}'s bounds are changed when the destination TreeNode of this {@code VisualComponentConnection}
     *is a descendant of a TreeNode that get's expanded or collapsed
     *@see VisualComponentConnectionUtils#moveBelowConnectionsBy
     */
    protected JComponent originalDest;

    /**A {@code JComponent} indicating the source of this {@code VisualComponentConnection}.
     *This {@code JComponent}'s bounds are changed when the source TreeNode of this {@code VisualComponentConnection}
     *is a descendant of a TreeNode that get's expanded or collapsed or when the
     * {@code JTree} component gets moved(scroll occurs)
     *@see VisualComponentConnectionUtils#moveBelowConnectionsBy
     *@see VisualComponentConnectionUtils#updateConnectionsVisibility
     */
    protected JComponent source;

    /**A {@code JComponent} indicating the destination of this {@code VisualComponentConnection}.
     *This {@code JComponent}'s bounds are changed when the destination TreeNode of this {@code VisualComponentConnection}
     *is a descendant of a TreeNode that get's expanded or collapsed or when the
     *{@code JTree} component gets moved(scroll occurs)
     *@see VisualComponentConnectionUtils#moveBelowConnectionsBy
     *@see VisualComponentConnectionUtils#updateConnectionsVisibility*/
    protected JComponent dest;

/* Test fields  */
    /**An instance of {@code TreePath} indicating the TreePath with last Component
     the source of this {@code VisualComponentConnection}*/
    protected TreePath sourcePath;

    /**An instance of {@code TreePath} indicating the TreePath with last Component
     the destination of this {@code VisualComponentConnection}*/
    protected TreePath destPath;

    
    /**
     * Constructs the default {@code VisualComponentConnection}.
     * @param source JComponent
     * @param dest JComponent
     */
    public VisualComponentConnection(JComponent source, JComponent dest) {
        this(source, dest, ConnectLine.LINE_ARROW_NONE, Color.BLACK);
    }

    /**
     * Constructs a {@code VisualComponentConnection} with the specified arrow and color.
     * @param source JComponent
     * @param dest JComponent
     * @param lineArrow int
     * @param lineColor Color
     */
    public VisualComponentConnection(JComponent source, JComponent dest, int lineArrow, Color lineColor) {
        this(source, dest, lineArrow, VisualComponentConnectionLineProperties.CONNECT_LINE_TYPE_RECTANGULAR, lineColor,true);
    }

    /**
     * Constructs a {@code VisualComponentConnection} with the specified arrow, line type and color.
     * @param source JComponent
     * @param dest JComponent
     * @param lineArrow int
     * @param lineType int
     * @param lineColor Color
     */
    public VisualComponentConnection(JComponent source, JComponent dest, int lineArrow, int lineType, Color lineColor, boolean visible) {
        this.source = source;
        this.originalSource = source;
        this.dest = dest;
        this.originalDest = dest;
        this.visibilityProp = new VisualComponentConnectionVisibilityProperties(visible,visible,!visible,!visible);
        this.lineProp = new VisualComponentConnectionLineProperties(lineArrow,lineType,lineColor);
    }

    /**
     * Constructs a {@code VisualComponentConnection} between two <strong>imaginary</strong> Compoments
     * with same bounds as <code>sourceRect</code> and <code>destRect</code>.
     * By default the source and dest are constructed as instances of JLabel.
     * @param sourceRect Rectangle
     * @param destRect Rectangle
     * @param lineArrow int
     * @param lineType int
     * @param lineColor Color
     */
    public VisualComponentConnection(Rectangle sourceRect, Rectangle destRect, int lineArrow,int lineType, Color lineColor,boolean visible){
        this.source = new JLabel();
        this.source.setBounds(sourceRect);
        this.originalSource = this.source;
        this.dest = new JLabel();
        this.dest.setBounds(destRect);
        this.originalDest = this.dest;
        this.visibilityProp = new VisualComponentConnectionVisibilityProperties(visible,visible,!visible,!visible);
        this.lineProp = new VisualComponentConnectionLineProperties(lineArrow,lineType,lineColor);
     }



    /**
     * Overrides parent's paint(). It resets clip to draw connecting line
     * between components and set the clip back.
     * @param g Graphics
     */
    @Override
    public void paint(Graphics g) {
        super.paint(g);
        Graphics2D g2d = (Graphics2D) g;
        calculateLine();
        if ((line != null)&&(visibilityProp.isVisible())&&(!visibilityProp.isCollapsed())) {
            Shape oldClip = g2d.getClip();
            g2d.setClip(getLineBounds());
            g2d.setColor(lineProp.getLineColor());
            line.paint(g2d);
            g2d.setClip(oldClip);
        }
    }

    /**
     * Calculates the starting and the end point of this {@code VisualComponentConnection}
     * depending on it's {@code source} and {@code dest} bounds
     */
    protected void calculateLine() {
        Rectangle rSource = source.getBounds();
        Rectangle rDest = dest.getBounds();
        if (rSource.intersects(rDest)) {
            line = null;
            return;
        }

        boolean xIntersect = (rSource.x <= rDest.x && rSource.x + rSource.width >= rDest.x)
            || (rDest.x <= rSource.x && rDest.x + rDest.width >= rSource.x);
        boolean yIntersect = rSource.y <= rDest.y && rSource.y + rSource.height >= rDest.y
            || (rDest.y <= rSource.y && rDest.y + rDest.height >= rSource.y);

        if (xIntersect) {
            int y1;
            int y2;
            int x1 = rSource.x + rSource.width / 2;
            int x2 = rDest.x + rDest.width / 2;
            if (rSource.y + rSource.height <= rDest.y) {
                //source higher
                y1 = rSource.y + rSource.height;
                y2 = rDest.y;
            }
            else {
                y1 = rSource.y;
                y2 = rDest.y + rDest.height;
            }
            line = new ConnectLine(new Point(x1, y1), new Point(x2, y2), ConnectLine.LINE_TYPE_RECT_2BREAK, ConnectLine.LINE_START_VERTICAL, lineProp.getLineArrow());
            if (lineProp.getLineType() == VisualComponentConnectionLineProperties.CONNECT_LINE_TYPE_SIMPLE) {
                line.setLineType(ConnectLine.LINE_TYPE_SIMPLE);
            }
        }
        else if (yIntersect) {
            int y1 = rSource.y + rSource.height / 2;
            
            int y2 = rDest.y + rDest.height / 2;
            
            int x1;
            int x2;
            if (rSource.x + rSource.width <= rDest.x) {
                x1 = rSource.x + rSource.width;
                x2 = rDest.x;
            }
            else {
                x1 = rSource.x;
                x2 = rDest.x + rDest.width;
            }
            line = new ConnectLine(new Point(x1, y1), new Point(x2, y2), ConnectLine.LINE_TYPE_RECT_2BREAK, ConnectLine.LINE_START_HORIZONTAL, lineProp.getLineArrow());
            if (lineProp.getLineType() == VisualComponentConnectionLineProperties.CONNECT_LINE_TYPE_SIMPLE) {
                line.setLineType(ConnectLine.LINE_TYPE_SIMPLE);
            }
        }
        else {
            int y1;
            int y2;
            int x1;
            int x2;
            if (rSource.y + rSource.height <= rDest.y) {
                //source higher
                y1 = rSource.y + rSource.height / 2;
                y2 = rDest.y + rDest.height/2;
                if (rSource.x + rSource.width <= rDest.x) {
                    x1 = rSource.x + rSource.width;
                }
                else {
                    x1 = rSource.x;
                }
                x2 = rDest.x;// + rDest.width / 2;
            }
            else {
                y1 = rSource.y + rSource.height / 2;
                y2 = rDest.y + rDest.height/2;
                if (rSource.x + rSource.width <= rDest.x) {
                    x1 = rSource.x + rSource.width;
                }
                else {
                    x1 = rSource.x;
                }
                x2 = rDest.x;// + rDest.width / 2;
            }
            line = new ConnectLine(new Point(x1, y1), new Point(x2, y2), ConnectLine.LINE_TYPE_RECT_2BREAK, ConnectLine.LINE_START_HORIZONTAL, lineProp.getLineArrow());
            if (lineProp.getLineType() == VisualComponentConnectionLineProperties.CONNECT_LINE_TYPE_SIMPLE) {
                line.setLineType(ConnectLine.LINE_TYPE_SIMPLE);
            }
        }
    }

    protected Rectangle getLineBounds() {
        int add = 10;
        int maxX = Math.max(line.getP1().x, line.getP2().x);
        int minX = Math.min(line.getP1().x, line.getP2().x);
        int maxY = Math.max(line.getP1().y, line.getP2().y);
        int minY = Math.min(line.getP1().y, line.getP2().y);

        Rectangle res = new Rectangle(minX - add, minY - add, maxX - minX + 2 * add, maxY - minY + 2 * add);
        return res;
    }


     /**
     * @return An instance of <code>Color</code> class, indicating the color
     * of this <code>VisualComponentConnection</code>
     */
    public Color getLineColor() {
        return lineProp.getLineColor();
    }

    /**
     * Sets the <code>color</code> field of this <code>VisualComponentConnection</code> to <code>c</code>
     * @param lineColor An instance of <code>Color</code> class
     */
    public void setLineColor(Color lineColor) {
        lineProp.setLineColor(lineColor);
    }

    /**
     * @return An integer, indicating the line type of this <code>VisualComponentConnection</code>
     */
    public int getLineType() {
        return lineProp.getLineType();
    }

    /**
     * Sets the line type of this <code>VisualComponentConnection</code> to <code>type</code>
     * @param lineType An integer indicating the line type
     */
    public void setLineType(int lineType) {
        lineProp.setLineType(lineType);
    }

    public int getLineArrow() {
        return lineProp.getLineArrow();
    }

    public void setLineArrow(int lineArrow) {
        lineProp.setLineArrow(lineArrow);
    }
    /**
     * @return The source Component of this <code>VisualComponentConnection</code>
     */
    public JComponent getSource(){
        return source;
    }

    /**
     * Sets the source component of this <code>VisualComponentConnection</code> to a <code>JLabel</code>
     * with same bounds as <code>rect</code>
     * @param rect An instance of <code>Rectangle</code>
     */
    public void setSource(Rectangle rect){
        this.source = new JLabel();
        this.source.setBounds(rect);
    }

    /**
     * Sets the <code>x</code> coordinate of this <code>VisualComponentConnection</code>'s source
     * Component to <code>incX</code>
     * @param incX An integer indicating the new x coordinate
     */
    public void setSourceX(int incX){
        Rectangle rSource = this.getSource().getBounds();
        rSource.setRect(incX, rSource.y, rSource.width, rSource.height);
        this.setSource(rSource);
    }

    /**
     * Moves this <code>VisualComponentConnection</code>'s source Component by the <code>x,y</code>
     * coordinates indicated by <code>incPoint</code>
     * @param incPoint An instance of <code>Point</code> class indicating the
     * <code>x,y</code> coordinates
     */
    public void moveSourceBy(Point incPoint){
        this.moveSourceBy(incPoint.x,incPoint.y);
    }

    /**
     * Moves this <code>VisualComponentConnection</code>'s source Component for <code>(incX,incY)</code>
     * on the <code>(x-axis,y-axis)</code> of the coordinate system
     * @param incX An integer indicating the distance for the <code>x-axis</code>
     * @param incY An integer indicating the distance for the <code>y-axis</code>
     */
    public void moveSourceBy(int incX, int incY){
        Rectangle rSource = this.getSource().getBounds();
        rSource.setRect(rSource.x+incX, rSource.y+incY, rSource.width, rSource.height);
        this.setSource(rSource);
    }

    /**
     * 
     * @return The destination Component of this <code>VisualComponentConnection</code>
     */
    public JComponent getDest(){
        return dest;
    }

    /**
     * Sets the destination component of this <code>VisualComponentConnection</code> to a <code>JLabel</code>
     * with same bounds as <code>rect</code>
     * @param rect An instance of <code>Rectangle</code>
     */
    public void setDest(Rectangle rect){
        this.dest = new JLabel();
        this.dest.setBounds(rect);
    }

    /**
     * Sets the <code>x</code> coordinate of this <code>VisualComponentConnection</code>'s destination
     * Component to <code>incX</code>
     * @param incX An integer indicating the new x coordinate
     */
    public void setDestX(int incX){
        Rectangle rDest = this.getDest().getBounds();
        rDest.setRect(incX, rDest.y, rDest.width, rDest.height);
        this.setDest(rDest);
    }

    /**
     * Moves this <code>VisualComponentConnection</code>'s destination Component by the <code>x,y</code>
     * coordinates indicated by <code>incPoint</code>
     * @param incPoint An instance of <code>Point</code> class indicating the
     * <code>x,y</code> coordinates
     */
    public void moveDestBy(Point incPoint){
        this.moveDestBy(incPoint.x,incPoint.y);
    }

    /**
     * Moves this <code>VisualComponentConnection</code>'s destination Component for <code>(incX,incY)</code>
     * on the <code>(x-axis,y-axis)</code> of the coordinate system
     * @param incX An integer indicating the distance for the <code>x-axis</code>
     * @param incY An integer indicating the distance for the <code>y-axis</code>
     */
    public void moveDestBy(int incX, int incY){
        Rectangle rDest = this.getDest().getBounds();
        rDest.setRect(rDest.x+incX, rDest.y+incY, rDest.width, rDest.height);
        this.setDest(rDest);
    }

    public void setSourceDest(Rectangle sourceRect, Rectangle destRect, boolean visible){
        if (sourceRect != null){
            this.source = new JLabel();
            this.source.setBounds(sourceRect);
        }
        if(destRect != null){
            this.dest = new JLabel();
            this.dest.setBounds(destRect);
        }
        this.visibilityProp.setVisible(visible);
    }

    /**
     * Moves each of this <code>VisualComponentConnection</code>'s source and destination components
     * by the <code>(x,y)</code> coordinates indicated by <code>incSource and incDest</code>
     * points
     * @param incSource An instance of <code>Point</code>
     * @param incDest An instance of <code>Point</code>
     */
    public void moveConnectionBy (Point incSource, Point incDest){
        this.moveConnectionBy(incSource.x, incSource.y, incDest.x, incDest.y);
    }

    /**
     * Moves each of this <code>VisualComponentConnection</code>'s source and destination components
     * by <code>(incSourceX,incSourceY) and (incDestX,incDestY)</code> respectively
     * @param incSourceX The <code>x</code> coordinate for the source Component
     * @param incSourceY The <code>y</code> coordinate for the source Component
     * @param incDestX The <code>x</code> coordinate for the destination Component
     * @param incDestY The <code>y</code> coordinate for the destination Component
     */
    public void moveConnectionBy(int incSourceX, int incSourceY, int incDestX, int incDestY){
        this.moveSourceBy(incSourceX, incSourceY);
        this.moveDestBy(incDestX, incDestY);
    }

    public void setOriginalSource(Rectangle rSource){
        this.originalSource = new JLabel();
        this.originalSource.setBounds(rSource);
    }

    public JComponent getOriginalSource(){
        return this.originalSource;
    }

    public void moveOriginalSourceBy(int incX, int incY){
        Rectangle rSource = this.getOriginalSource().getBounds();
        rSource.setRect(rSource.x+incX, rSource.y+incY, rSource.width, rSource.height);
        this.setOriginalSource(rSource);
    }

    public void setOriginalDest(Rectangle rDest){
        this.originalDest = new JLabel();
        this.originalDest.setBounds(rDest);
    }

    public JComponent getOriginalDest(){
        return this.originalDest;
    }

    public void moveOriginalDestBy(int incX, int incY){
        Rectangle rDest = this.getOriginalDest().getBounds();
        rDest.setRect(rDest.x+incX, rDest.y+incY, rDest.width, rDest.height);
        this.setOriginalDest(rDest);
    }

    public void resetToOriginal(){
        this.source = this.originalSource;
        this.dest = this.originalDest;
    }

    public void setPaths(TreePath sourcePath, TreePath destPath){
        this.sourcePath = sourcePath;
        this.destPath = destPath;
    }

    public TreePath getSourcePath(){
        return this.sourcePath;
    }

    public TreePath getDestPath(){
        return this.destPath;
    }

    public VisualComponentConnectionVisibilityProperties getVisibilityProperties(){
        return this.visibilityProp;
    }

    public void setVisibilityProperties(VisualComponentConnectionVisibilityProperties visibilityProp){
        this.visibilityProp = visibilityProp;
    }

    public VisualComponentConnectionLineProperties getLineProperties(){
        return this.lineProp;
    }

    public void setLineProperties(VisualComponentConnectionLineProperties lineProp){
        this.lineProp = lineProp;
    }

    public void print(){
        String sourceName = (String) ((DefaultMutableTreeNode)sourcePath.getLastPathComponent()).getUserObject();
        String destName = (String) ((DefaultMutableTreeNode)destPath.getLastPathComponent()).getUserObject();
        System.out.println("Source: " + sourceName+"("+source.getBounds().x+","+source.getBounds().y+")"+
                "\tDestination: " + destName+"("+dest.getBounds().x+","+dest.getBounds().y+")");
    }
    @Override
    public String toString(){
        String sourceName = (String) ((DefaultMutableTreeNode)sourcePath.getLastPathComponent()).getUserObject();
        String destName = (String) ((DefaultMutableTreeNode)destPath.getLastPathComponent()).getUserObject();
        return new String(sourceName+"-->"+destName);
    }
}
