package org.ow2.choreos.gui.visualComponentConnection;

import java.util.ArrayList;
import java.util.LinkedList;

import org.ow2.choreos.gui.interfaceMappingsVisualizer.components.MappingsPanel;

/**
 * This class is used for holding history of alterations(deletion/addition) of
 * {@link VisualComponentConnection} components that occured on a {@link MappingsPanel}.
 */
public class VisualComponentConnectionManagementHistory {

    public static final int DELETING_EXISTING_CONNECTION = 0;
    public static final int CANCELING_ADDITION_OF_CONNECTION = 1;
    public static final int ADDING_NEW_CONNECTION = 2;
    LinkedList<HistoryPacket> undoStack;
    LinkedList<HistoryPacket> redoStack;
    MappingsPanel container;

    public VisualComponentConnectionManagementHistory(MappingsPanel container) {
        this.container = container;
        undoStack = new LinkedList<HistoryPacket>();
        redoStack = new LinkedList<HistoryPacket>();
    }

    public void ClearHistory() {
        clearRedoStack();
        clearUndoStack();
        UpdateUndoRedoButtons();
    }

    public void ExecuteUndo() {
        HistoryPacket packet = (HistoryPacket) undoStack.pop();
        redoStack.push(packet);
        UpdateUndoRedoButtons();
        /**EXECUTE UNDO**/
        switch (packet.getId()) {
            case DELETING_EXISTING_CONNECTION:
                addToExistingConnections(packet.getConnection());
                break;
            case CANCELING_ADDITION_OF_CONNECTION:
                addToAddingConnections(packet.getConnection());
                break;
            case ADDING_NEW_CONNECTION:
                addToDeletingConnections(packet.getConnection());
                break;
            default:
                break;
        }

        container.repaint();
    }

    public void ExecuteRedo() {
        HistoryPacket packet = (HistoryPacket) redoStack.pop();
        undoStack.push(packet);
        UpdateUndoRedoButtons();
        /**EXECUTE REDO**/
        switch (packet.getId()) {
            case DELETING_EXISTING_CONNECTION:
                addToDeletingConnections(packet.getConnection());
                break;
            case CANCELING_ADDITION_OF_CONNECTION:
                addToDeletingConnections(packet.getConnection());
                break;
            case ADDING_NEW_CONNECTION:
                addToAddingConnections(packet.getConnection());
                break;
            default:
                break;
        }
        container.repaint();
    }

    private void addToExistingConnections(VisualComponentConnection connection) {
        container.getDeletedMappings().remove(connection);
        connection.setLineColor(container.getLineColor());
        container.getExistingMappings().add(connection);
    }

    private void addToDeletingConnections(VisualComponentConnection connection) {
        if (container.getExistingMappings().remove(connection)) {
            connection.setLineColor(container.getDeletedLineColor());
            container.getDeletedMappings().add(connection);
        }
        container.getAddedMappings().remove(connection);
    }

    private void addToAddingConnections(VisualComponentConnection connection) {
        if (!VisualComponentConnectionUtils.connectionIsContained(container, VisualComponentConnectionUtils.ALL_CONNECTIONS, connection)) {
            connection.setLineColor(container.getAddedLineColor());
            container.getAddedMappings().add(connection);
        }
    }

    public void pushIntoUndo(int iD, VisualComponentConnection connection) {
        undoStack.push(new HistoryPacket(iD, connection));
        UpdateUndoRedoButtons();
    }

    public void pushIntoRedo(int iD, VisualComponentConnection connection) {
        redoStack.push(new HistoryPacket(iD, connection));
        UpdateUndoRedoButtons();
    }

    private void UpdateUndoRedoButtons() {
        if (undoStack.isEmpty()) {
            container.getUndoButton().setEnabled(false);
            container.getUndoButton().setToolTipText(null);
        } else {
            container.getUndoButton().setEnabled(true);
            HistoryPacket packet = (HistoryPacket) undoStack.peek();
            container.getUndoButton().setToolTipText("<html>Undo: " + packet.getMessage()+"</html>");
        }
        
        if (redoStack.isEmpty()) {
            container.getRedoButton().setEnabled(false);
            container.getRedoButton().setToolTipText(null);
        } else {
            container.getRedoButton().setEnabled(true);
            HistoryPacket packet = (HistoryPacket) redoStack.peek();
            container.getRedoButton().setToolTipText("<html>Redo: " + packet.getMessage()+"</html>");
        }
    }

    public LinkedList<HistoryPacket> getRedoStack() {
        return redoStack;
    }

    public void clearRedoStack() {
        redoStack.clear();
        this.UpdateUndoRedoButtons();
    }

    public LinkedList<HistoryPacket> getUndoStack() {
        return undoStack;
    }

    public void clearUndoStack() {
        undoStack.clear();
        this.UpdateUndoRedoButtons();
    }

    public ArrayList<VisualComponentConnection> getCanceledAndDeletedConnections() {
        ArrayList<VisualComponentConnection> connections = new ArrayList<VisualComponentConnection>();
        for (int i = 0; i < undoStack.size(); i++) {
            HistoryPacket packet = (HistoryPacket) undoStack.get(i);
            if (packet.getId() == CANCELING_ADDITION_OF_CONNECTION
                    || packet.getId() == ADDING_NEW_CONNECTION)
                if(!VisualComponentConnectionUtils.connectionIsContained(container, VisualComponentConnectionUtils.ALL_CONNECTIONS, packet.getConnection())
                    && !connections.contains(packet.getConnection()))
                    connections.add(packet.getConnection());
        }
        for (int i = 0; i < redoStack.size(); i++) {
            HistoryPacket packet = (HistoryPacket) redoStack.get(i);
            if (packet.getId() == CANCELING_ADDITION_OF_CONNECTION
                    || packet.getId() == ADDING_NEW_CONNECTION)
                if(!VisualComponentConnectionUtils.connectionIsContained(container, VisualComponentConnectionUtils.ALL_CONNECTIONS, packet.getConnection())
                    && !connections.contains(packet.getConnection()))
                    connections.add(packet.getConnection());
        }
        return connections;
    }
}

class HistoryPacket {

    private int iD;
    private String message;
    private VisualComponentConnection connection;

    public HistoryPacket(int iD, VisualComponentConnection connection) {
        this.iD = iD;
        this.connection = connection;
        if (iD == VisualComponentConnectionManagementHistory.DELETING_EXISTING_CONNECTION) {
            this.message = "Deleting Mapping between:<br>" + connection.getSourcePath().toString() + "<br>and<br>" + connection.getDestPath().toString();
        } else if (iD == VisualComponentConnectionManagementHistory.ADDING_NEW_CONNECTION) {
            this.message = "Adding Mapping between:<br>" + connection.getSourcePath().toString() + "<br>and<br>" + connection.getDestPath().toString();
        } else if (iD == VisualComponentConnectionManagementHistory.CANCELING_ADDITION_OF_CONNECTION) {
            this.message = "Cancel Adding Mapping between:<br>" + connection.getSourcePath().toString() + "<br>and<br>" + connection.getDestPath().toString();
        }
    }

    public VisualComponentConnection getConnection() {
        return connection;
    }

    public void setConnection(VisualComponentConnection connection) {
        this.connection = connection;
    }

    public int getId() {
        return iD;
    }

    public void setId(int iD) {
        this.iD = iD;
    }

    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    void print() {
        System.out.println(message);
    }
}
