package org.ow2.choreos.gui.visualComponentConnection;

import java.awt.Point;
import java.awt.Rectangle;
import java.util.ArrayList;

import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.interfaceMappingsToJTrees.TreeUtils;
import org.ow2.choreos.gui.interfaceMappingsVisualizer.components.MappingsPanel;

/**
 * This class implements useful static methods used for {@link VisualComponentConnection}
 * managing. <ul>Such as:<li>Finding certain VisualComponentConnections inside a {@link MappingsPanel}
 * <li>Changing the visibility or collapsion properties of certain VisualComponentConnections
 * <li>Moving certain VisualComponentConnections inside a {@link MappingsPanel} <li>Checking
 * if a VisualComponentConnection is contained in a {@link MappingsPanel}<ul>
 *
 * @see VisualComponentConnection
 * @see MappingsPanel
 */
public class VisualComponentConnectionUtils {

    public static final int SOURCE = 0;
    public static final int DESTINATION = 1;
    public static final int ALL_CONNECTIONS = 0;
    public static final int EXISTING_AND_FOR_DELETION_CONNECTIONS = 1;
    public static final int EXISTING_AND_FOR_ADDITION_CONNECTIONS = 2;
    public static final int EXISTING_CONNECTIONS = 3;
    public static final int FOR_ADDITION_CONNECTIONS = 4;
    public static final int FOR_DELETION_CONNECTIONS = 5;
    public static final int ALL_AND_CANCELED_CONNECTIONS = 6;
    private static int endPoint;

    /**
     * Updates the visibility of all {@link VisualComponentConnection}s' sources or
     * destinations(depending on <code>endPoint</code>) contained in
     * {@link MappingsPanel container}. This method simply checks
     * if a <code>VisualComponentConnection</code>'s specified endpoint should be visible after
     * some Component movement has occured.
     * @param container An instance of {@code MappingsPanel}
     * @param endPoint An integer indicating which <code>endpoint<code> should be used
     * for checking its visibility for each {@code VisualComponentConnection}
     * <ul>Valid IDs are:
     * <li>VisualComponentConnectionUtils.SOURCE
     * <li>VisualComponentConnectionUtils.DESTINATION
     */
    public static void updateConnectionsVisibility(MappingsPanel container, int endPoint) {
        Point oldTreeLocation = null, newTreeLocation = null;
        Rectangle visibleRect = null;

        ArrayList<VisualComponentConnection> connections = new ArrayList<VisualComponentConnection>(container.getExistingMappings());
        connections.addAll(container.getAddedMappings());
        connections.addAll(container.getDeletedMappings());
        connections.addAll(container.getMappingsFromHistory());
        if (endPoint == SOURCE) {
            oldTreeLocation = container.getSourceTreeLocation();
            newTreeLocation = container.getSourceTree().getLocation();
            visibleRect = container.getSourceTreeScrollPane().getViewport().getBounds();
        } else if (endPoint == DESTINATION) {
            oldTreeLocation = container.getDestTreeLocation();
            newTreeLocation = container.getDestTree().getLocation();
            visibleRect = container.getDestTreeScrollPane().getViewport().getBounds();
        }

        Point maxVisible = new Point(visibleRect.width - newTreeLocation.x, visibleRect.height - newTreeLocation.y);

        int yDiff = newTreeLocation.y - oldTreeLocation.y;

        for (int i = 0; i < connections.size(); i++) {
            switch (endPoint) {
                case DESTINATION:
                    connections.get(i).moveDestBy(0, yDiff);
                    Rectangle rDest = connections.get(i).getOriginalDest().getBounds();

                    if (((rDest.y + rDest.height / 2) < (-newTreeLocation.y)) || ((rDest.y + rDest.height / 2) > maxVisible.y)) {
                        connections.get(i).getVisibilityProperties().setVisibleDest(false);
                    } else {
                        connections.get(i).getVisibilityProperties().setVisibleDest(true);
                    }
                    
                    if ((rDest.x < -newTreeLocation.x) || (rDest.x > maxVisible.x)) {
                        connections.get(i).setDestX(container.getDestScrollPaneLocation().x);
                    }
                    else{
                        connections.get(i).setDestX(newTreeLocation.x + rDest.x + container.getDestScrollPaneLocation().x);
                    }
                    break;
                case SOURCE:
                    connections.get(i).moveSourceBy(0, yDiff);
                    Rectangle rSource = connections.get(i).getOriginalSource().getBounds();

                    if (((rSource.y + rSource.height / 2) < (-newTreeLocation.y)) || ((rSource.y + rSource.height / 2) > maxVisible.y)) {
                        connections.get(i).getVisibilityProperties().setVisibleSource(false);
                    } else {
                        connections.get(i).getVisibilityProperties().setVisibleSource(true);
                    }
                    
                    if (((rSource.x + rSource.width) < -newTreeLocation.x) || ((rSource.x + rSource.width) > maxVisible.x)) {
                        connections.get(i).setSourceX(container.getSourceScrollPaneLocation().x + visibleRect.width - rSource.width);
                    }
                    else{
                        connections.get(i).setSourceX(newTreeLocation.x + rSource.x + container.getSourceScrollPaneLocation().x);
                    }
                    break;
                default:
                    break;
            }
        }
    }

    /**
     * Moves the {@link VisualComponentConnection connection} components within {@code container} with end points to the ones
     * specified by <code>endPoint</code>) below <code>aNode</code> by {@code yDiff}
     *
     * @param container An instance of {@link MappingsPanel}
     * @param endPoint An integer indicating which end point should be used.
     * <ul>Valid IDs are:
     * <li>TreeUtils.SOURCE
     * <li>TreeUtils.DESTINATION
     * </ul>
     * @param yDiff An integer specifing the distace the endpoints are to be moved
     * @param aNode A TreeNode
     */
    public static void moveBelowConnectionsBy(MappingsPanel container, int endPoint, int yDiff, DefaultMutableTreeNode aNode) {
        VisualComponentConnectionUtils.endPoint = endPoint;

        for (int i = 0; i < aNode.getChildCount(); i++) {
            moveDescendantsConnectionsBy(container, (DefaultMutableTreeNode) aNode.getChildAt(i), yDiff);
        }

        DefaultMutableTreeNode nextNode = aNode.getNextSibling();
        while (nextNode != null) {
            moveDescendantsConnectionsBy(container, nextNode, yDiff);
            nextNode = nextNode.getNextSibling();
        }

        moveBelowAncestorsConnectionsBy(container, (DefaultMutableTreeNode) aNode.getParent(), yDiff);
    }

    /**
     * Moves the {@link VisualComponentConnection connection} components with source or destination
     * nodes to any of <code>aNodes</code>'s next siblings and ancestors by <code>yDiff</code>
     *
     * @param container An instance of {@link MappingsPanel}
     * @param yDiff The distance the source or destination of the connections are to be moved
     * @param aNode The parent TreeNode
     */
    public static void moveBelowAncestorsConnectionsBy(MappingsPanel container, DefaultMutableTreeNode aNode, int yDiff) {
        if (aNode != null) {
            DefaultMutableTreeNode nextNode = aNode.getNextSibling();
            while (nextNode != null) {
                moveDescendantsConnectionsBy(container, nextNode, yDiff);
                nextNode = nextNode.getNextSibling();
            }

            moveBelowAncestorsConnectionsBy(container, (DefaultMutableTreeNode) aNode.getParent(), yDiff);
        }
    }

    /**
     * Moves the {@link VisualComponentConnection connection} components with source or destination
     * nodes to <code>aNode</code> and any of its descendants  by <code>yDiff</code>
     *
     * @param container An instance of {@link MappingsPanel}
     * @param yDiff The distance the source or destination of the connections are to be moved
     * @param aNode The parent TreeNode
     */
    public static void moveDescendantsConnectionsBy(MappingsPanel container, DefaultMutableTreeNode aNode, int yDiff) {
        ArrayList<VisualComponentConnection> foundConnections = null;
        switch (endPoint) {
            case DESTINATION:
                foundConnections = VisualComponentConnectionUtils.findConnectionsWith(container, ALL_AND_CANCELED_CONNECTIONS, null, aNode);
                for (int i = 0; i < foundConnections.size(); i++) {
                    foundConnections.get(i).moveDestBy(0, yDiff);
                    foundConnections.get(i).moveOriginalDestBy(0, yDiff);
                }
                break;
            case SOURCE:
                foundConnections = VisualComponentConnectionUtils.findConnectionsWith(container, ALL_AND_CANCELED_CONNECTIONS, aNode, null);
                for (int i = 0; i < foundConnections.size(); i++) {
                    foundConnections.get(i).moveSourceBy(0, yDiff);
                    foundConnections.get(i).moveOriginalSourceBy(0, yDiff);
                }
                break;
            default:
                break;
        }
        if (aNode.getChildCount() == 0) {
            return;
        } else {
            for (int i = 0; i < aNode.getChildCount(); i++) {
                VisualComponentConnectionUtils.moveDescendantsConnectionsBy(container, (DefaultMutableTreeNode) aNode.getChildAt(i), yDiff);
            }
        }
    }

    /**
     * Sets the collapsion property of all {@link VisualComponentConnection connection} components
     * with {@code endPoint} nodes
     * to any of </code>aNode's</code> descendant nodes to <code>state</code>
     *
     * @param container An instance of {@link MappingsPanel}
     * @param endPoint An integer indicating which <code>end point</code> should be used.
     * <ul>Valid IDs are:
     * <li>SOURCE
     * <li>DESTINATION
     * </ul>
     * @param state The collapsion state
     * @param aNode The parent TreeNode
     */
    public static void setDescendantsConnectionCollapsion(MappingsPanel container, int endPoint, boolean state, DefaultMutableTreeNode aNode) {
        JTree aTree = null;
        if (endPoint == SOURCE) {
            aTree = container.getSourceTree();
        } else if (endPoint == DESTINATION) {
            aTree = container.getDestTree();
        }

        if (aTree != null) {
            TreePath aPath = TreeUtils.ConvertTreeNodeToPath(aNode);
            for (int i = 0; i < aNode.getChildCount(); i++) {
                DefaultMutableTreeNode iChild = (DefaultMutableTreeNode) aNode.getChildAt(i);

                ArrayList<VisualComponentConnection> foundConnections = new ArrayList<VisualComponentConnection>();
                if (aTree.isExpanded(aPath)) {
                    switch (endPoint) {
                        case SOURCE:
                            foundConnections = VisualComponentConnectionUtils.findConnectionsWith(container, ALL_AND_CANCELED_CONNECTIONS, iChild, null);
                            for (int j = 0; j < foundConnections.size(); j++) {
                                foundConnections.get(j).getVisibilityProperties().setCollapsedSource(state);
                            }
                            break;
                        case DESTINATION:
                            foundConnections = VisualComponentConnectionUtils.findConnectionsWith(container, ALL_AND_CANCELED_CONNECTIONS, null, iChild);
                            for (int j = 0; j < foundConnections.size(); j++) {
                                foundConnections.get(j).getVisibilityProperties().setCollapsedDest(state);
                            }
                            break;
                        default:
                            break;
                    }
                    setDescendantsConnectionCollapsion(container, endPoint, state, iChild);
                }
            }
        }
    }

    /**
     * Finds and returns all {@code VisualComponentConnection} components
     * with <code>source</code> as their source node and <code>dest</code>
     * as their destination node.
     * <P>
     * <strong>Note</strong>: By setting one of {@code source or dest} null, this method searchs
     * for {@code VisualComponentConnection}s with the non-null value as one of it's endpoints
     *
     * @param container An instance of {@link MappingsPanel}
     * @param choice An Integer indicating which of the {@code VisualComponentConnection} lists of {@code container}
     * should be searched.<ul>Valid choices are:
     * <li>ALL_CONNECTIONS
     * <li>EXISTING_AND_FOR_DELETION_CONNECTIONS
     * <li>EXISTING_AND_FOR_ADDITION_CONNECTIONS
     * <li>EXISTING_CONNECTIONS
     * <li>FOR_ADDITION_CONNECTIONS
     * <li>FOR_DELETION_CONNECTIONS
     * </ul>
     * @param source The source TreeNode
     * @param dest The destination TreeNode
     * @return An array of {@code VisualComponentConnection} components.
     * If no {@code VisualComponentConnection}s were found returns an empty array
     */
    public static ArrayList<VisualComponentConnection> findConnectionsWith(MappingsPanel container, int choice, DefaultMutableTreeNode source, DefaultMutableTreeNode dest) {
        ArrayList<VisualComponentConnection> foundConnections = new ArrayList<VisualComponentConnection>();
        ArrayList<VisualComponentConnection> connections = new ArrayList<VisualComponentConnection>();

        if (choice != FOR_ADDITION_CONNECTIONS && choice != FOR_DELETION_CONNECTIONS) {
            connections.addAll(container.getExistingMappings());
        }
        if (choice == EXISTING_AND_FOR_ADDITION_CONNECTIONS || choice == FOR_ADDITION_CONNECTIONS || choice == ALL_CONNECTIONS || choice == ALL_AND_CANCELED_CONNECTIONS) {
            connections.addAll(container.getAddedMappings());
        }
        if (choice == EXISTING_AND_FOR_DELETION_CONNECTIONS || choice == FOR_DELETION_CONNECTIONS || choice == ALL_CONNECTIONS || choice == ALL_AND_CANCELED_CONNECTIONS) {
            connections.addAll(container.getDeletedMappings());
        }
        if (choice == ALL_AND_CANCELED_CONNECTIONS) {
            connections.addAll(container.getMappingsFromHistory());
        }
        if (connections != null) {
            if ((source != null) && (dest != null)) {
                for (int i = 0; i <
                        connections.size(); i++) {
                    DefaultMutableTreeNode iDest =
                            (DefaultMutableTreeNode) connections.get(i).getDestPath().getLastPathComponent();
                    DefaultMutableTreeNode iSource =
                            (DefaultMutableTreeNode) connections.get(i).getSourcePath().getLastPathComponent();
                    if ((source.equals(iSource)) && (dest.equals(iDest))) {
                        foundConnections.add(connections.get(i));
                    }

                }
            } else if ((dest == null) && (source != null)) {
                for (int i = 0; i <
                        connections.size(); i++) {
                    DefaultMutableTreeNode iSource =
                            (DefaultMutableTreeNode) connections.get(i).getSourcePath().getLastPathComponent();
                    if (source.equals(iSource)) {
                        foundConnections.add(connections.get(i));
                    }

                }
            } else if ((source == null) && (dest != null)) {
                for (int i = 0; i <
                        connections.size(); i++) {
                    DefaultMutableTreeNode iDest =
                            (DefaultMutableTreeNode) connections.get(i).getDestPath().getLastPathComponent();
                    if (dest.equals(iDest)) {
                        foundConnections.add(connections.get(i));
                    }
                }
            }
        }
        return foundConnections;
    }

    /**
     * Returns {@code true} if the specified {@link VisualComponentConnection connection} component is
     * contained in one of the {@code VisualComponentConnection} lists of {@link MappingsPanel
     * container} depending from {@code choice}. Returns {@code false} otherwise.
     *
     * @param container An instance of {@link MappingsPanel}
     * @param choice An Integer indicating which of the {@code VisualComponentConnection} lists of {@code container}
     * should be checked.<ul>Valid choices are:
     * <li>ALL_CONNECTIONS
     * <li>EXISTING_AND_FOR_DELETION_CONNECTIONS
     * <li>EXISTING_AND_FOR_ADDITION_CONNECTIONS
     * <li>EXISTING_CONNECTIONS
     * <li>FOR_ADDITION_CONNECTIONS
     * <li>FOR_DELETION_CONNECTIONS
     * </ul>
     * @param connection The {@code VisualComponentConnection} component
     * @return {@code true} if contained, {@code false} otherwise
     */
    public static boolean connectionIsContained(MappingsPanel container, int choice, VisualComponentConnection connection) {
        if (connection != null) {
            return !VisualComponentConnectionUtils.findConnectionsWith(container, choice,
                    (DefaultMutableTreeNode) connection.getSourcePath().getLastPathComponent(),
                    (DefaultMutableTreeNode) connection.getDestPath().getLastPathComponent()).isEmpty();
        } else {
            return false;
        }
    }
}
