package org.ow2.choreos.queryEngineFacade;

import java.util.List;
import java.util.ArrayList;
import java.io.Serializable;

import org.ow2.choreos.serviceRepresentation.functional.Operation;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * Stores advanced query information.
 * An advanced query consists of functional and non-functional constraints, plus requirements of dependencies on other services.
 * 
 * @author pilios
 *
 */
@XmlRootElement
public class AdvancedQuery implements Serializable{

	private List<OperationConstraints> operationsConstraints;
	private List<QualityPropertyConstraints> qualityPropertiesConstraints;
	private String RQs;
	
	/**
	 * Creates a complete AdvancedQuery object.
	 * Formal parameters must conform to some rules.For more information see {@link #checkWellFormedNess()}
	 * 
	 * @param operationsConstraints Functional constraints (concerning service operations).
	 * @param qualityPropertiesConstraints Non-functional constraints (concerning quality properties of services).
	 * @param RQs Requirements of dependencies on other services.
	 * Requirements are separated by commas.
	 */
	public AdvancedQuery(List<OperationConstraints> operationsConstraints, List<QualityPropertyConstraints> qualityPropertiesConstraints, String RQs){
		this.operationsConstraints = operationsConstraints;
		this.qualityPropertiesConstraints = qualityPropertiesConstraints;
		this.RQs = RQs;
	}
	
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 */
	public AdvancedQuery(){
		
	}
	
	/**
	 * Returns a list of names of service operations.
	 * 
	 * @return A list of names of service operations.
	 */
	public List<String> getOperationNames(){
	
		List<String> operationsNames = new ArrayList<String>();
		int operationsConstraintsSize = operationsConstraints.size();
		
		for(int i = 0; i < operationsConstraintsSize; i++){
			OperationConstraints opConstrs = operationsConstraints.get(i);
			operationsNames.add(opConstrs.getName());
		}
		
		return operationsNames;
	}
	
	/** 
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Sets functional constraints.
	 * 
	 * @param operationsConstraints A list of functional constraints.
	 */
	public void setOperationsConstraints(List<OperationConstraints> operationsConstraints){
		this.operationsConstraints = operationsConstraints;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Sets non-functional constraints. 
	 * 
	 * @param qualityPropertiesConstraints A list of functional constraints.
	 */
	public void setQualityPropertiesConstraints(List<QualityPropertyConstraints> qualityPropertiesConstraints){
		this.qualityPropertiesConstraints = qualityPropertiesConstraints;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Sets requirements of dependencies on other services. 
	 * 
	 * @param RQs Requirements of dependencies on other services.
	 */
	public void setRQs(String RQs){
		this.RQs = RQs;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Returns functional constraints.
	 *   
	 * @return A list of functional constraints.
	 */
	public List<OperationConstraints> getOperationsConstraints(){
		return operationsConstraints;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Returns non-functional constraints. 
	 *  
	 * @return A list of non-functional constraints.
	 */
	public List<QualityPropertyConstraints> getQualityPropertiesConstraints(){
		return qualityPropertiesConstraints;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Returns requirements of dependencies on other services. 
	 * 
	 * @return Requirements of dependencies on other services. 
	 */
	public String getRQs(){
		return RQs;
	}
	
	/**
	 * <pre>
	 * Checks whether the AdvancedQuery object is well-formed or not.
	 * If the AdvancedQuery object is not well-formed, an exception is thrown.
	 * 
	 * Well-formedness rules:
	 * 
	 * List&lt{@link OperationConstraints}> operationsConstraints:
	 * 
	 * 	Must not be null or empty.An operationsConstraints list item, say opConstr, must not be null.
	 * 	opConstr.name must not be null.
	 * 	For opConstr.inParamNames and opConstr.inParamTypes the following
	 * 	must hold: 
	 * 		it is ok to be both null, but if not, they must not be empty and must be of the same size.Additionally,
	 * 		items in relative positions in the two lists are thought to be related under name-type relation, and
	 * 		they must not be both null.
	 * 	The same must hold for opConstr.outParamNames and opConstr.outParamTypes.
	 * 
	 * List&lt{@link QualityPropertyConstraints}> qualityPropertiesConstraints:
	 * 
	 * 	It is ok to be null.A qualityPropertiesConstraints item, say qualityPropertyConstraints, must not be null.
	 * 	qualityPropertyConstraints.name must not be null.
	 * 	Either qualityPropertyConstraints.valueDescription or qualityPropertyConstraints.valueFrom and 
	 * 	qualityPropertyConstraints.valueTo must be specified, and, if qualityPropertyConstraints.valueDescription 
	 * 	is specified, it must not be null.
	 * </pre>
	 * 
	 * @throws QueryEngineFacadeException If the AdvancedQuery object is not well-formed.
	 */
	public void checkWellFormedNess() throws QueryEngineFacadeException{
		
		List<OperationConstraints> operationsConstraints = getOperationsConstraints();
		if(operationsConstraints == null){
			throw (new QueryEngineFacadeException("AdvancedQuery object not well formed: operationsConstraints must not be null"));
		}
		
		int operationsConstraintsSize = operationsConstraints.size();
		if(operationsConstraintsSize == 0){
			throw (new QueryEngineFacadeException("AdvancedQuery object not well formed: operationsConstraints must not be empty"));
		}
		
		List<String> queryOperations = new ArrayList<String>();
		
		for(int i = 0; i < operationsConstraintsSize; i++){
			
			OperationConstraints opConstrs = operationsConstraints.get(i);
			if(opConstrs == null){
				throw (new QueryEngineFacadeException("AdvancedQuery object not well formed: operationConstraints must not be null"));
			}
			
			String opConstrsName = opConstrs.getName();
			if(opConstrsName == null){
				throw (new QueryEngineFacadeException("AdvancedQuery object not well formed: operationConstraints.name must not be null"));
			}
			
			queryOperations.add(opConstrs.getName());
			
			List<String> inParamNames = opConstrs.getInParamNames();
			List<String> inParamTypes = opConstrs.getInParamTypes();
			
			if(inParamNames == null && inParamTypes != null){
				throw (new QueryEngineFacadeException("AdvancedQuery object not well formed: operationConstraints.inParamNames and operationConstraints.inParamTypes must be of the same size"));
			}
			
			if(inParamNames != null && inParamTypes == null){
				throw (new QueryEngineFacadeException("AdvancedQuery object not well formed: operationConstraints.inParamNames and operationConstraints.inParamTypes must be of the same size"));
			}
			
			if(inParamNames != null && inParamTypes != null){
				
				int inParamNamesSize = inParamNames.size();
				int inParamTypesSize = inParamTypes.size();
				
				if(inParamNamesSize != inParamTypesSize){
					throw (new QueryEngineFacadeException("AdvancedQuery object not well formed: operationConstraints.inParamNames and operationConstraints.inParamTypes must be of the same size"));
				}
				
				if(inParamNamesSize == 0){
					throw (new QueryEngineFacadeException("AdvancedQuery object not well formed: operationConstraints.inParamNames and operationConstraints.inParamTypes must not be empty"));
				}
				
				for(int j = 0; j < inParamNamesSize; j++){
					if(inParamNames.get(j) == null && inParamTypes.get(j) == null){
						throw (new QueryEngineFacadeException("AdvancedQuery object not well formed: operationConstraints.inParamNames.get(i) and operationConstraints.inParamTypes.get(i) must not be both null"));
					}
				}
			}
			
			List<String> outParamNames = opConstrs.getOutParamNames();
			List<String> outParamTypes = opConstrs.getOutParamTypes();
			
			if(outParamNames == null && outParamTypes != null){
				throw (new QueryEngineFacadeException("AdvancedQuery object not well formed: operationConstraints.outParamNames and operationConstraints.outParamTypes must be of the same size"));
			}
			
			if(outParamNames != null && outParamTypes == null){
				throw (new QueryEngineFacadeException("AdvancedQuery object not well formed: operationConstraints.outParamNames and operationConstraints.outParamTypes must be of the same size"));
			}
			
			if(outParamNames != null && outParamTypes != null){
				
				int outParamNamesSize = outParamNames.size();
				int outParamTypesSize = outParamTypes.size();
				
				if(outParamNamesSize != outParamTypesSize){
					throw (new QueryEngineFacadeException("AdvancedQuery object not well formed: operationConstraints.outParamNames and operationConstraints.outParamTypes must be of the same size"));
				}
				
				if(outParamNamesSize == 0){
					throw (new QueryEngineFacadeException("AdvancedQuery object not well formed: operationConstraints.outParamNames and operationConstraints.outParamTypes must not be empty"));
				}
				
				for(int j = 0; j < outParamNamesSize; j++){
					if(outParamNames.get(j) == null && outParamTypes.get(j) == null){
						throw (new QueryEngineFacadeException("AdvancedQuery object not well formed: operationConstraints.outParamNames.get(i) and operationConstraints.outParamTypes.get(i) must not be both null"));
					}
				}
			}
				
		}
		
		List<QualityPropertyConstraints> qualityPropertiesConstraints = getQualityPropertiesConstraints();
		
		if(qualityPropertiesConstraints != null){
			int qualityPropertiesConstraintsSize = qualityPropertiesConstraints.size();
			
			for(int i = 0; i < qualityPropertiesConstraintsSize; i++){
				QualityPropertyConstraints qualityPropertyConstraints = qualityPropertiesConstraints.get(i);
				
				if(qualityPropertyConstraints == null){
					throw (new QueryEngineFacadeException("AdvancedQuery object not well formed: qualityPropertyConstraints must not be null"));
				}
				
				if(qualityPropertyConstraints.getName() == null){
					throw (new QueryEngineFacadeException("AdvancedQuery object not well formed: qualityPropertyConstraints.name must not be null"));
				}
				
				if(qualityPropertyConstraints.getHasValueDescription() == true && qualityPropertyConstraints.getValueDescription() == null){
					throw (new QueryEngineFacadeException("AdvancedQuery object not well formed: quality property value description must not be null"));
				}
			}
		}
	}
}
