/**
 * The ConcreteServices class is used to interface with UDA queries
 */
package org.ow2.choreos.queryEngineFacade;

import java.util.*;

import org.ow2.choreos.serviceRepresentation.functional.Operation;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInstance;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;

/**
 * <pre>
 * It Is created as a result to a query and contains a service interface and the matching 
 * between its operations and the list of operations' names given in the query.
 * </pre>
 * 
 * @author zarras
 * @author pilios
 */
public class ConcreteService {
	
	private List<String> queryOperations;
	private ServiceInterface retrievedInterface;
	private HashMap<String, OperationsList> query2retrievedOperationsMapping;

	
	/**
	 * Creates a complete ConcreteService object.
	 * 
	 * @param queryOperations A list of names of service operations.
	 * @param retrievedInterface The service interface that was retrieved as a result to a query.
	 */
	public ConcreteService(List<String> queryOperations, ServiceInterface retrievedInterface){
		
		this.queryOperations = queryOperations;
		this.retrievedInterface = retrievedInterface;
		
		query2retrievedOperationsMapping = new HashMap<String, OperationsList>();
		List<Operation> retrievedOperations = retrievedInterface.getOperations();
		
		for(int i=0; i < queryOperations.size(); i++){
			
			String queryOperation = queryOperations.get(i);
			List<Operation> targetOperations = new ArrayList<Operation>();
			
			for(int j=0; j < retrievedOperations.size(); j++) {
				
				Operation retrievedOperation = retrievedOperations.get(j); 
				
				if(retrievedOperation.getName().toLowerCase().contains(queryOperation.toLowerCase()))
					targetOperations.add(retrievedOperation);
				
			}
			
			OperationsList operationList = new OperationsList(targetOperations);
			query2retrievedOperationsMapping.put(queryOperation, operationList);
		
		}
		
	}
	
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 */
	public ConcreteService(){
		
	}
	
	/**
	 * Returns a list of operations that match an input operation name (query operation).
	 * 
	 * @param queryOperation An operation name.
	 * @return A list of operations.
	 */
	public List<Operation> getRetrievedOperations(String queryOperation){
		
		OperationsList operationList = query2retrievedOperationsMapping.get(queryOperation);
		
		return(operationList.getOperationList());
	}
	
	/**
	 * Returns the behavioral specification URI of the service interface.
	 * 
	 * @return The behavioral specification URI of the service interface.
	 */
	public String getBehavioralSpecificationURI(){
		String b = retrievedInterface.getBehavioralSpecification();
		String pieces[] = b.split(" "); 
		if(pieces.length < 3){
			System.err.println("There is no enactment information available");
			return null;
		}
		return pieces[2];
	}
	
	/**
	 * Returns the enactment URI of the service interface.
	 * 
	 * @return The enactment URI of the service interface.
	 */
	public String getEnactementURI(){
		String b = retrievedInterface.getBehavioralSpecification();
		String pieces[] = b.split(" ");
		if(pieces.length < 7){
			System.err.println("There is no enactment information available");
			return null;
		}
		return pieces[6];
	}
	
	/**
	 * Returns the URIs of the service instances that are represented by the service interface.
	 * 
	 * @return A list of URIs of the service instances that are represented by the service interface.
	 */
	public List<String> getServiceInstancesURIs(){
		List<String> result = new ArrayList<String>();
		List<ServiceInstance> retrievedInstances = retrievedInterface.getServiceInstances();
		
		if(retrievedInstances == null)
			return null;
		
		for(int i=0; i < retrievedInstances.size(); i++)
			result.add(retrievedInstances.get(i).getURI());
		return result;
	}
	
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Sets query operations' names.
	 * 
	 * @param queryOperations A list of names of query operations.
	 */
	public void setQueryOperations(List<String> queryOperations){
		this.queryOperations = queryOperations;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Sets the service interface.
	 * 
	 * @param serviceInterface The service interface.
	 */
	public void setServiceInterface(ServiceInterface serviceInterface){
		retrievedInterface = serviceInterface;
	}
	
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Sets the matching between the service interface operations and the list of operations' names given 
	 * in the query.
	 * 
	 * @param query2retrievedOperationsMapping The matching between the service interface operations and 
	 * the list of operations' names given in the query.
	 */
	public void setQuery2retrievedOperationsMapping(HashMap<String, OperationsList> query2retrievedOperationsMapping){
		this.query2retrievedOperationsMapping = query2retrievedOperationsMapping;
		
		for (Map.Entry<String, OperationsList> entry : this.query2retrievedOperationsMapping.entrySet()) {
		    String key = entry.getKey();
		    OperationsList value = entry.getValue();
		    
		}
		
	}
	
	/**
	 * Returns the query operations' names.
	 * 
	 * @return A list of names of query operations.
	 */
	public List<String> getQueryOperations(){
		return queryOperations;
	}
	
	/**
	 * Returns the service interface.
	 * 
	 * @return The service interface.
	 */
	public ServiceInterface getServiceInterface(){
		return retrievedInterface;
	}
	
	/**
	 * Returns the matching between the service interface operations and the list of operations' names given 
	 * in the query.
	 * 
	 * @return The matching between the service interface operations and the list of operations' names given 
	 * in the query.
	 */
	public HashMap<String, OperationsList> getQuery2retrievedOperationsMapping(){
		return query2retrievedOperationsMapping;
	}
	
}


