package org.ow2.choreos.queryEngineServiceDeployment;

import java.util.ArrayList;
import java.util.List;

import javax.ws.rs.POST;
import javax.ws.rs.Produces;
import javax.ws.rs.Consumes;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;

import org.ow2.choreos.abstractionBase.FunctionalAbstraction;
import org.ow2.choreos.abstractionBase.NonFunctionalAbstraction;
import org.ow2.choreos.queryEngine.QueryEngineController;
import org.ow2.choreos.queryEngineFacade.ConcreteService;
import org.ow2.choreos.queryEngineFacade.QueryEngineFacade;
import org.ow2.choreos.queryEngineFacade.SimpleQuery;
import org.ow2.choreos.queryEngineFacade.AdvancedQuery;
import org.ow2.choreos.queryEngineFacade.QueryEngineFacadeException;
import org.ow2.choreos.serviceRepresentation.functional.Operation;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInstance;

/**
 * <pre>
 * Acts as a <a href = 'https://en.wikipedia.org/wiki/Representational_state_transfer'>RESTful web service</a>.
 * Each of its operations can be called by a service consumer (client).
 * Provides operations that look up for services taking as input a service description.This can be:
 * 
 * -	A description in a specific query language, named Service Base Query Language (SBQL).
 * 
 * -	A simple description consisting of a list of names of service operations and a string of requirements
 *   	of dependencies on other services.
 *   
 * -	A more advanced query consisting of more detailed functional and non-functional constraints, as well
 *   	as a string of requirements of dependencies on other services.
 *   
 * Searching can be executed over service abstractions (functional and non-functional) or service instances.
 * In all cases the returned result is a javax.ws.rs.core.Response object that contains a {@link QueryEngineServiceResponse} object.
 * 
 * The server address a service client must connect to is <span style="color:green;">http://localhost:8080/servicedeployer</span>.
 * To connect with this class (service), the relative path <span style="color:green;">/queryengine</span> must be added, and a 
 * service operation's relative path must be added too, so that a complete path for establishing a connection to a specific operation 
 * is created.
 * Each service operation's relative path include a username and a password for authorization purposes.Thus, authorization is part 
 * of connection establishment and client does not need to post username and password as inputs to an operation.
 * 
 * Every operation returns a javax.ws.rs.core.Response object that contains a {@link QueryEngineServiceResponse} object.
 * If something goes wrong about network connection or service searching, the relevant error fields of QueryEngineResponse object 
 * are filled in properly.
 * Otherwise, the relevant information fields of QueryEngineResponse are filled properly.
 * 
 * </pre>
 * 
 * @author pilios
 *
 */

@Path("/queryengine")
public class QueryEngineService {
    
	/**
	 * <pre>
	 * Looks up for services, taking as input a SBQL query.
	 * The searching is executed over functional and non-functional service abstractions and the result contains
	 * functional and non-functional abstractions.
	 * </pre>
	 * 
	 * @param username Username provided during client's connection establishment.
	 * @param password Password provided during client's connection establishment.
	 * @param sbqlQuery SBQL query
	 * @return A javax.ws.rs.core.Response object that contains a {@link QueryEngineServiceResponse} object.
	 * The QueryEngineResponse object has its functional and non-functional abstractions fields filled.	
	 */
	@Path("SBQLQuery/{username}/{password}")
    @POST
    @Produces(MediaType.APPLICATION_XML)
	@Consumes(MediaType.APPLICATION_XML)
    public Response executeSBQLQuery(@PathParam("username") String username, @PathParam("password") String password, String sbqlQuery) {

		QueryEngineController qEngine = new QueryEngineController();
    	
    	qEngine.executeLookupQuery("", sbqlQuery);
    
    	ArrayList <FunctionalAbstraction> functionalAbstractions = qEngine.getRetrievedFunctionalAbstractions();
    	ArrayList <NonFunctionalAbstraction> nonFunctionalAbstractions = qEngine.getRetrievedNonFunctionalAbstractions();
    	
    	QueryEngineServiceResponse qeResponse = new QueryEngineServiceResponse(Status.OK, functionalAbstractions, nonFunctionalAbstractions);
    	
		return Response.ok(qeResponse).build();
    	
    }
	
	/**
	 * <pre>
	 * Looks up for services, taking as input a simple query.
	 * A simple query contains a list of names of operations and a string of requirements of dependencies on other services.
	 * The searching is executed over functional service abstractions and the result contains concrete services.
	 * </pre>
	 * 
	 * @param username Username provided during client's connection establishment.
	 * @param password Password provided during client's connection establishment.
	 * @param query Simple query, containing a list of names of operations and a string of requirements of dependencies on other services.
	 * @return A javax.ws.rs.core.Response object that contains a {@link QueryEngineServiceResponse} object.
	 * The QueryEngineResponse object has its concrete services field filled. 		
	 */
	@Path("SimpleQueryOverAbstractions/{username}/{password}")
    @POST
    @Produces(MediaType.APPLICATION_XML)
	@Consumes(MediaType.APPLICATION_XML)
    public Response executeSimpleQueryOverAbstractions(@PathParam("username") String username, @PathParam("password") String password, SimpleQuery query) {
		
		QueryEngineFacade qEngineFacade = new QueryEngineFacade();
		
		List<ConcreteService> concreteServices;
		QueryEngineServiceResponse qeResponse;
		
		try {
			concreteServices = qEngineFacade.executeSimpleQueryOverAbstractions("", query);
			qeResponse = new QueryEngineServiceResponse(Status.OK, concreteServices);
		} catch (QueryEngineFacadeException qefEx) {
			qeResponse = new QueryEngineServiceResponse(Status.OK, qefEx.getMessage());
		}
    	
		return Response.ok(qeResponse).build();
    	
    }
	
	/**
	 * <pre>
	 * Looks up for services, taking as input a simple query.
	 * A simple query contains a list of names of operations and a string of requirements of dependencies on other services.
	 * The searching is executed over service instances and the result contains concrete services.
	 * </pre>
	 * 
	 * @param username Username provided during client's connection establishment.
	 * @param password Password provided during client's connection establishment.
	 * @param query Simple query, containing a list of names of operations and a string of requirements of dependencies on other services.
	 * @return A javax.ws.rs.core.Response object that contains a {@link QueryEngineServiceResponse} object.
	 * The QueryEngineResponse object has its concrete services field filled. 		
	 */
	@Path("SimpleQueryOverInstances/{username}/{password}")
    @POST
    @Produces(MediaType.APPLICATION_XML)
	@Consumes(MediaType.APPLICATION_XML)
    public Response executeSimpleQueryOverInstances(@PathParam("username") String username, @PathParam("password") String password, SimpleQuery query) {
		
		QueryEngineFacade qEngineFacade = new QueryEngineFacade();
			
		List<ConcreteService> concreteServices;
		QueryEngineServiceResponse qeResponse;
		
		try {
			concreteServices = qEngineFacade.executeSimpleQueryOverInstances("", query);
			qeResponse = new QueryEngineServiceResponse(Status.OK, concreteServices);
		} catch (QueryEngineFacadeException qefEx) {
			qeResponse = new QueryEngineServiceResponse(Status.OK, qefEx.getMessage());
		}
    	
		return Response.ok(qeResponse).build();
    	
    }
	
	/**
	 * <pre>
	 * Looks up for services, taking as input an advanced query.
	 * An advanced query contains functional and non-functional constraints, and a string of requirements of 
	 * dependencies on other services.
	 * The searching is executed over functional and non-functional service abstractions and the result contains 
	 * concrete services.
	 * </pre>
	 * 
	 * @param username Username provided during client's connection establishment.
	 * @param password Password provided during client's connection establishment.
	 * @param query Advanced query, containing functional and non-functional constraints, and a string of 
	 * requirements of dependencies on other services.
	 * @return A javax.ws.rs.core.Response object that contains a {@link QueryEngineServiceResponse} object.
	 * The QueryEngineResponse object has its concrete services field filled. 		
	 */
	@Path("AdvancedQueryOverAbstractions/{username}/{password}")
    @POST
    @Produces(MediaType.APPLICATION_XML)
	@Consumes(MediaType.APPLICATION_XML)
    public Response executeAdvancedQueryOverAbstractions(@PathParam("username") String username, @PathParam("password") String password, AdvancedQuery query) {
		
		QueryEngineFacade qEngineFacade = new QueryEngineFacade();
		
		List<ConcreteService> concreteServices;
		QueryEngineServiceResponse qeResponse;
		
		try {
			concreteServices = qEngineFacade.executeAdvancedQueryOverAbstractions("", query);
			qeResponse = new QueryEngineServiceResponse(Status.OK, concreteServices);
		} catch (QueryEngineFacadeException qefEx) {
			qeResponse = new QueryEngineServiceResponse(Status.OK, qefEx.getMessage());
		}
    	
		return Response.ok(qeResponse).build();
    	
    }
	
	/**
	 * <pre>
	 * Looks up for services, taking as input an advanced query.
	 * An advanced query contains functional and non-functional constraints, and a string of requirements of 
	 * dependencies on other services.
	 * The searching is executed over service instances and the result contains concrete services.
	 * </pre>
	 * 
	 * @param username Username provided during client's connection establishment.
	 * @param password Password provided during client's connection establishment.
	 * @param query Advanced query, containing functional and non-functional constraints, and a string of 
	 * requirements of dependencies on other services.
	 * @return A javax.ws.rs.core.Response object that contains a {@link QueryEngineServiceResponse} object.
	 * The QueryEngineResponse object has its concrete services field filled. 		
	 */
	@Path("AdvancedQueryOverInstances/{username}/{password}")
    @POST
    @Produces(MediaType.APPLICATION_XML)
	@Consumes(MediaType.APPLICATION_XML)
    public Response executeAdvancedQueryOverInstances(@PathParam("username") String username, @PathParam("password") String password, AdvancedQuery query) {
		
		QueryEngineFacade qEngineFacade = new QueryEngineFacade();
		
		List<ConcreteService> concreteServices;
		QueryEngineServiceResponse qeResponse;
		
		try {
			concreteServices = qEngineFacade.executeAdvancedQueryOverInstances("", query);
			qeResponse = new QueryEngineServiceResponse(Status.OK, concreteServices);
		} catch (QueryEngineFacadeException qefEx) {
			qeResponse = new QueryEngineServiceResponse(Status.OK, qefEx.getMessage());
		}
		
		return Response.ok(qeResponse).build();
    	
    }
    
}

