package org.ow2.choreos.queryEngineServiceDeployment;

import java.util.List;
import java.util.ArrayList;
import org.ow2.choreos.abstractionBase.FunctionalAbstraction;
import org.ow2.choreos.abstractionBase.NonFunctionalAbstraction;
import org.ow2.choreos.queryEngineFacade.ConcreteService;
import javax.ws.rs.core.Response.Status;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * Contains information that is a response to a query about services.
 * It also contains information about possible errors, regarding network
 * connection or service searching.
 * 
 * @author pilios
 *
 */

@XmlRootElement
public class QueryEngineServiceResponse {

	private Status httpStatus;
	private String errorMessage;
	private List<FunctionalAbstraction> functionalAbstractions;
	private List<NonFunctionalAbstraction> nonFunctionalAbstractions;
	private List<ConcreteService> concreteServices;
	
	/**
	 * Creates a QueryEngineResponse object.
	 * 
	 * @param status HTTP status code, indicating transmission errors.
	 * @param fa Service functional abstractions.
	 * @param nfa Service non-functional abstractions.
	 */
	public QueryEngineServiceResponse( Status status, List<FunctionalAbstraction> fa, List<NonFunctionalAbstraction> nfa ) {
		
		httpStatus = status;
		functionalAbstractions = fa;
		nonFunctionalAbstractions = nfa;
		concreteServices = null;
		errorMessage = null;
	}
	
	
	/**
	 * Creates a QueryEngineResponse object.
	 * 
	 * @param status HTTP status code, indicating possible transmission errors.
	 * @param cConcreteServices Concrete services.
	 */
	public QueryEngineServiceResponse( Status status, List<ConcreteService> concreteServices) {
		
		httpStatus = status;
		functionalAbstractions = null;
		nonFunctionalAbstractions = null;
		this.concreteServices = concreteServices;
		errorMessage = null;
	}
	
	/**
	 * Creates a QueryEngineResponse object.
	 * Used when errors have occured.
	 * 
	 * @param status HTTP status code, indicating transmission errors.
	 * @param errorMessage Error message.
	 */
	public QueryEngineServiceResponse( Status status, String errorMessage) {
		
		httpStatus = status;
		this.errorMessage = errorMessage;
		functionalAbstractions = null;
		nonFunctionalAbstractions = null;
		concreteServices = null;
		errorMessage = null;
	}
	

	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 */
	public QueryEngineServiceResponse() {
		
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Sets HTTP status.
	 * 
	 * @param status HTTP status code.
	 */
	public void setHttpStatus(Status status) {
		httpStatus = status;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Sets error message.
	 * 
	 * @param errorMessage Error message.
	 */
	public void setErrorMessage(String errorMessage) {
		this.errorMessage = errorMessage;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Sets functional abstractions.
	 * 
	 * @param fa Functional abstractions.
	 */
	public void setFunctionalAbstractions( List<FunctionalAbstraction> fa ) {
		functionalAbstractions = fa;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Sets non-functional abstractions.
	 * 
	 * @param nfa Non-functional abstractions.
	 */
	public void setNonFunctionalAbstractions( List<NonFunctionalAbstraction> nfa ) {
		nonFunctionalAbstractions = nfa;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Sets concrete services.
	 * 
	 * @param concreteServices Concrete services.
	 */
	public void setConcreteServices( List<ConcreteService> concreteServices ) {
		this.concreteServices = concreteServices;
	}
	
	/**
	 * Returns HTTP status code, indicating possible transmission errors.
	 * 
	 * @return HTTP status code.
	 */
	public Status getHttpStatus() {
		return httpStatus;
	}
	
	/**
	 * Returns error message, indicating possible errors regarding service searching or posted query structure.
	 * 
	 * @return Error message.
	 */
	public String getErrorMessage() {
		return errorMessage;
	}
	
	/**
	 * Returns a list of service functional abstractions.
	 * 
	 * @return A list of service functional abstractions.
	 */
	public List<FunctionalAbstraction> getFunctionalAbstractions() {
		return functionalAbstractions;
	}
	
	/**
	 * Returns a list of service non-functional abstractions.
	 * 
	 * @return A list of service non-functional abstractions.
	 */
	public List<NonFunctionalAbstraction> getNonFunctionalAbstractions() {
		return nonFunctionalAbstractions;
	}
	
	/**
	 * Returns a list of concrete services.
	 * 
	 * @return A list of concrete services.
	 */
	public List<ConcreteService> getConcreteServices() {
		return concreteServices;
	}
	
}

