package org.ow2.choreos.schemaRepresentation;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.classInfo.ClassInfo;
import org.ow2.choreos.dataStructures.interval.Interval1D;
import org.ow2.choreos.dataStructures.list.listManipulation.ListTransformations;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.parser.freeTextDocuments.Tokenizer;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.schemaRepresentation.xml.Annotation;
import org.ow2.choreos.schemaRepresentation.xml.Component;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;


/**
 * 
 * @author dathanas
 * This class models a 'Schema' without sticking to an XML schema.
 */
public abstract class Schema {


	//Variables.
	public final static String schemaTagName = "schema";


	private Map<String, String> tagAttributes;

	private String schemaKind;//e.g., XML.

	private String globalQualifier;//The 'String' is the qualifier of the 'targetNamespace' that exist in the 'definitions' tag of the WSDL file, e.g. xmlns:prod="http://example.org/prod", String = prod.
	private boolean hasGlobalQualifier;

	private List<String> schemaQualifiers;//The 'String' are the qualifiers of the 'targetNamespace' that exist in the the schema tag.
	private boolean hasSchemaQualifiers;

	private Map<String, String> remainingQualifierNamespaces;//These are the 'qualifier' - 'namespace' that exist in the schema tag for other targetNamespaces and not the namespace of this schema. 
	private boolean hasRemainingQualifierNamespaces;

	private String targetNamespace;
	private boolean hasTargetNamespace;


	private int myOrderID = 0;//This is the order in which the schema appears in its schema document.

	private int myChildrenOrderID = 0;//This is the order in which the components of the schema appear in this schema.


	private int size = 1;//This is the total number of the building blocks (without following the references).

	private List<Component> globalComponents;
	private boolean hasGlobalComponents;

	private boolean hasUnmaterializedComponents;

	private List<Schema> importedSchemas;
	private boolean hasImportedSchemas;

	private Annotation annotation;
	private boolean hasAnnotation;

	private Interval1D scope;

	private Object parent;


	private String constraintViolation;
	private boolean hasConstraintViolation;


	//Constructor.
	public Schema( String qualifier, Map<String, String > attributes ){

		hasGlobalComponents = false;
		hasImportedSchemas = false;
		hasUnmaterializedComponents = false;
		hasConstraintViolation = false;
		hasRemainingQualifierNamespaces = false;
		hasAnnotation = false;


		if( qualifier != null && ! qualifier.isEmpty() ){

			globalQualifier = qualifier;

			hasGlobalQualifier = true;
		}

		else hasGlobalQualifier = false;


		if( attributes != null && ! attributes.isEmpty() ) tagAttributes = attributes;


		String cTargetNamespace = MapGetter.getValueByKeyToLowerCase( "targetNamespace", tagAttributes );

		//if( checkConstraint( cTargetNamespace ) == null ){
		if( cTargetNamespace != null && ! cTargetNamespace.isEmpty() ){

			targetNamespace = cTargetNamespace;

			hasTargetNamespace = true;
		}

		else hasTargetNamespace = false;


		parseNamespaceQualifiers();


		printNamespaces();


		PrintMessages.print( PrintMessages.xmlPrint, hasConstraintViolation(), this );
	}

	//Methods used by the constructor.
	private void parseNamespaceQualifiers(){

		if( tagAttributes != null ){

			for ( Enumeration<String> e = ( (Hashtable<String, String>) tagAttributes ).keys(); e.hasMoreElements(); ){

				String attributeName = e.nextElement();

				String attributeValue = tagAttributes.get( attributeName );


				if( Tokenizer.getPrefix( attributeName, ":" ) != null && Tokenizer.getPrefix( attributeName, ":" ).equals( "xmlns" ) ){

					String qualifier = Tokenizer.getSuffix( attributeName, ":" );

					String namespace = attributeValue;

					if( hasTargetNamespace && targetNamespace.equals( namespace ) ){

						if( ! hasSchemaQualifiers ){

							schemaQualifiers = new ArrayList<String>();

							hasSchemaQualifiers = true;
						}

						schemaQualifiers.add( qualifier );
					}

					else{

						if( ! hasRemainingQualifierNamespaces ){

							remainingQualifierNamespaces = new Hashtable<String, String>();

							hasRemainingQualifierNamespaces = true;
						}

						remainingQualifierNamespaces.put( qualifier, namespace );
					}
				}
			}
		}
	}

	protected void printNamespaces(){

		PrintMessages.stdPrintLn( PrintMessages.xmlPrint, "Schema - " + targetNamespace );

		if( hasGlobalQualifier ) PrintMessages.stdPrintLn( PrintMessages.xmlPrint, "targetNamespace= " + targetNamespace + " - globalQualifier= " + globalQualifier );

		for( int i = 0; hasSchemaQualifiers && i < schemaQualifiers.size(); ++i ) PrintMessages.stdPrintLn( PrintMessages.xmlPrint, "targetNamespace= " + targetNamespace + " - qualifier= " + schemaQualifiers.get( i ) );


		if( hasRemainingQualifierNamespaces )

			for ( Enumeration<?> enumeration = ( (Hashtable<String, String>) remainingQualifierNamespaces ).keys(); enumeration.hasMoreElements(); ){

				String cqualifier = (String) enumeration.nextElement();

				PrintMessages.stdPrintLn( PrintMessages.xmlPrint, "remainingQualifier= " + cqualifier + " - namespace= " + remainingQualifierNamespaces.get( cqualifier ) );
			}
	}

	public String getName(){

		return getTagName();
	}


	//Constraint checkers.
	private String checkConstraint( String targetNamespace ){

		return targetNamespace == null || ( targetNamespace != null && targetNamespace.isEmpty() ) ? ObjectConstraintsManipulation.setEmptyAttribute( this, "targetNamespace" ) : null;
	}


	//Abstract methods.
	abstract protected boolean isComponentKind( Object object );

	abstract protected boolean isSchemaKind( Object object );

	abstract protected String getComponentKinds();

	abstract protected String getSchemaKinds();

	abstract public String getSchemaRepresentationModel();

	abstract public String getSchemaDefinitionNamespace();

	public abstract String getTagName();

	abstract public String getTag();

	abstract public KindDescr getKindDescr();

	abstract public KindDescr getKindDescr( String tagName );


	//Setters.
	public String setConstraintViolation( String cConstraintViolation ){

		if( ! hasConstraintViolation && cConstraintViolation != null ){

			constraintViolation = cConstraintViolation;

			hasConstraintViolation = true;
		}

		return constraintViolation;
	}

	protected void setMyOrderID( int id ){

		myOrderID = id;
	}

	public void setScope( Interval1D cScope ){

		scope = cScope;
	}

	public void setParent( Object cParent ){

		parent = cParent;
	}


	private void add( Component component ){

		if( component.getSchemaRepresentationModel().equals( getSchemaRepresentationModel() ) ){

			if( ! hasGlobalComponents ){
	
				globalComponents = new ArrayList<Component>();
	
				hasGlobalComponents = true;
			}
	
			hasUnmaterializedComponents = true;
	
			globalComponents.add( component );


			size += component.getSize();


			SchemaRepresentationController.setMyOrderID( component, ++myChildrenOrderID );
		}

		else ObjectConstraintsManipulation.setConnectionError ( this, component, "Its component does NOT have the SAME schema representation model" );
	}

	private void add( Schema importedSchema ){

		if( importedSchema.getSchemaRepresentationModel().equals( getSchemaRepresentationModel() ) ){

			if( ! hasImportedSchemas ){
	
				importedSchemas = new ArrayList<Schema>();
	
				hasImportedSchemas = true;
			}
	
			hasUnmaterializedComponents = true;
	
			importedSchemas.add( importedSchema );


			size += importedSchema.getSize();


			importedSchema.setMyOrderID( ++myChildrenOrderID );
		}

		else ObjectConstraintsManipulation.setConnectionError ( this, importedSchema, "Its imported schema does NOT have the SAME schema representation model" );
	}

	private void set( Annotation cAnnotation ){

		if( ! hasAnnotation ){

			annotation = cAnnotation;

			hasAnnotation = true;


			size += cAnnotation.getSize();


			SchemaRepresentationController.setMyOrderID( cAnnotation, ++myChildrenOrderID );
		}

		else ObjectConstraintsManipulation.setConnectionError ( this, cAnnotation, "Its annotation has already been set" );
	}

	public Object add( Object object ){

		if( ObjectConstraintsManipulation.setContainmentError( this, object ) == null ){

			if( isComponentKind( object ) ) add( (Component) object );

			else if( isSchemaKind( object ) ) add( (Schema) object );

			else if( XMLSchemaRepresentationController.isAnnotationKind( object ) ) set( (Annotation) object );

			else{

				String message1 = "Valid content:\n1. The components: " + getComponentKinds();

				String message2 = "\n2. The schemas: " + getSchemaKinds();

				String message3 = "\n3. Annotation.";

				ObjectConstraintsManipulation.setConnectionError ( this, object, message1 + message2 + message3 );
			}
		}


		return this;
	}

	public void connect(){

		Map<String, String> qualifierNamespaces = null;

		Map<String, Object> indexGlobalComponentsWithoutTargetNamespaces = null;

		Map<String, Object> indexGlobalComponentsWithTargetNamespaces = null;


		if( hasUnmaterializedComponents ){

			PrintMessages.print( PrintMessages.connectionXMLWithXMLPrint, "** START - Make Indices **" );

			qualifierNamespaces = makeIndexQualifierNamespaces( qualifierNamespaces );

			indexGlobalComponentsWithoutTargetNamespaces = makeIndexGlobalComponents( indexGlobalComponentsWithoutTargetNamespaces, "makeIndexGlobalComponentsWithoutTargetNamespaces" );

			indexGlobalComponentsWithTargetNamespaces = makeIndexGlobalComponents( indexGlobalComponentsWithTargetNamespaces, "makeIndexGlobalComponentsWithTargetNamespaces" );

			PrintMessages.print( PrintMessages.connectionXMLWithXMLPrint, "** END - Make Indices **" );


			connect( qualifierNamespaces, indexGlobalComponentsWithoutTargetNamespaces, indexGlobalComponentsWithTargetNamespaces );
		}
	}

	public void connect( Map<String, String> qualifierNamespaces, Map<String, Object> indexGlobalComponentsWithoutTargetNamespaces, Map<String, Object> indexGlobalComponentsWithTargetNamespaces ){

		PrintMessages.print( PrintMessages.connectionXMLWithXMLPrint, "START - connectXMLWithXML: " + getTag() + " - hasUnmaterializedComponents= " + hasUnmaterializedComponents() + " hasConstraintViolation= " + hasConstraintViolation );


		if( hasUnmaterializedComponents && ! hasConstraintViolation ){

			for( int i = 0; hasGlobalComponents && i < globalComponents.size(); ++i ){

				globalComponents.get( i ).connect( makeIndexRemainingGlobalComponentsWithoutTargetNamespaces( globalComponents.get( i ) ), qualifierNamespaces, indexGlobalComponentsWithoutTargetNamespaces, indexGlobalComponentsWithTargetNamespaces, getSchemaDefinitionNamespace() );


				if( globalComponents.get( i ).hasUnmaterializedReferencedComponents() ){

					if( globalComponents.get( i ).hasConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, globalComponents.get( i ) );

					return;
				}
			}

			for( int i = 0; hasImportedSchemas && i < importedSchemas.size(); ++i ){

				importedSchemas.get( i ).connect( qualifierNamespaces, indexGlobalComponentsWithoutTargetNamespaces, indexGlobalComponentsWithTargetNamespaces );


				if( importedSchemas.get( i ).hasUnmaterializedComponents() ){

					if( importedSchemas.get( i ).hasConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, importedSchemas.get( i ) );

					return;
				}
	        }

			hasUnmaterializedComponents = false;
		}


		PrintMessages.print( PrintMessages.connectionXMLWithXMLPrint, "END - connectXMLWithXML: " + getTag() + " - hasUnmaterializedComponents= " + hasUnmaterializedComponents() + " hasConstraintViolation= " + hasConstraintViolation );
	}

	protected void setSchemaKind( String cSchemaKind ){

		schemaKind = cSchemaKind;
	}

	public Map<String, String> makeIndexQualifierNamespaces( Map<String, String> index ){ 

		for( int i = 0; hasImportedSchemas && i < importedSchemas.size(); ++i ) index = importedSchemas.get( i ).makeIndexQualifierNamespaces( index );//I lately put the qualifiers of the imported schema in order to keep the local qualifiers of a schema in case there are synonymous global qualifiers.

		for( int i = 0; hasSchemaQualifiers && i < schemaQualifiers.size(); ++i ) index = add( index, schemaQualifiers.get( i ), targetNamespace );

		if( hasRemainingQualifierNamespaces )

			for ( Enumeration<String> e = ( (Hashtable<String, String>) remainingQualifierNamespaces ).keys(); e.hasMoreElements(); ){
	
				String qualifier = e.nextElement();

				String namespace = remainingQualifierNamespaces.get( qualifier );

				index = add( index, qualifier, namespace );
			}

		if( hasGlobalQualifier ) index = add( index, globalQualifier, targetNamespace );


		return index;
	}

	public Map<String, String> makeIndexNamespaceQualifiers( Map<String, String> index ){

		for( int i = 0; hasImportedSchemas && i < importedSchemas.size(); ++i ) index = importedSchemas.get( i ).makeIndexQualifierNamespaces( index );

		if( hasGlobalQualifier ) index = add( index, targetNamespace, globalQualifier );


		return index;
	}

	private Map<String, String> add( Map<String, String> index, String key, String value ){

		if( index == null ) index = new Hashtable<String, String>();

		if( key != null && ! index.containsKey( key ) ) index.put( key, value );


		return index;
	}

	public Map<String, Object> makeIndexGlobalComponents( Map<String, Object> indexComponents, String indexKind ){

		for( int i = 0; hasImportedSchemas && i < importedSchemas.size(); ++i ) indexComponents = importedSchemas.get( i ).makeIndexGlobalComponents( indexComponents, indexKind );

		for( int i = 0; hasGlobalComponents && i < globalComponents.size(); ++i ){

			Component component = globalComponents.get( i );

			String componentName = component.getName();

			String componentKind = ClassInfo.getClass( component );


			String key = null;

			if( indexKind.equals( "makeIndexGlobalComponentsWithTargetNamespaces" ) && hasTargetNamespace ) key = componentName + "," + componentKind + "," + targetNamespace;

			else if( indexKind.equals( "makeIndexGlobalComponentsWithoutTargetNamespaces" ) ) key = componentName + "," + componentKind;


			indexComponents = add( indexComponents, key, component );
		}


		return indexComponents;
	}

	private Map<String, Object> add( Map<String, Object> indexComponents, String key, Object component ){

		if( indexComponents == null ) indexComponents = new Hashtable<String, Object>();

		if( key != null && ! indexComponents.containsKey( key ) ) indexComponents.put( key, component );


		return indexComponents;
	}

	public Map<String, Object> makeIndexRemainingGlobalComponentsWithoutTargetNamespaces( Object component ){

		Map<String, Object> indexComponents = new Hashtable<String, Object>();


		for( int i = 0; hasGlobalComponents && i < globalComponents.size(); ++i ){

			if( ! globalComponents.get( i ).equals( component ) ){

				String componentName = globalComponents.get( i ).getName();

				String componentKind = ClassInfo.getClass( globalComponents.get( i ) );

				indexComponents = add( indexComponents, componentName + "," + componentKind, globalComponents.get( i ) );
			}
		}


		return indexComponents;
	}


	//Getters.
	public boolean hasConstraintViolation(){

		return hasConstraintViolation;
	}

	public String getConstraintViolation(){

		return constraintViolation;
	}

	public boolean hasFinalConstraintViolation(){

		return getFinalConstraintViolation() != null ? true : false;
	}

	public String getFinalConstraintViolation(){//It accumulates the constraints that are not called by the constructor.

		if( hasConstraintViolation ) return constraintViolation;


		//I did not find any 'checkConstraint()' method.


		for( int i = 0; hasImportedSchemas && i < importedSchemas.size(); ++i ) if( importedSchemas.get( i ).hasFinalConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, importedSchemas.get( i ) );

		for( int i = 0; hasGlobalComponents && i < globalComponents.size(); ++i ) if( globalComponents.get( i ).hasFinalConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, globalComponents.get( i ) );

		if( hasAnnotation && annotation.hasConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, annotation );


		return constraintViolation;
	}

	public boolean hasGlobalQualifier(){

		return hasGlobalQualifier;
	}

	public String getGlobalQualifier(){

		return globalQualifier;
	}

	public boolean hasSchemaQualifiers(){

		return hasSchemaQualifiers;
	}

	public List<String> getSchemaQualifiers(){

		return schemaQualifiers;
	}

	public boolean hasGlobalComponents(){

		return hasGlobalComponents;
	}

	public List<Component> getGlobalComponents(){

		return globalComponents;
	}

	public boolean hasUnmaterializedComponents(){

		return hasUnmaterializedComponents;
	}

	public boolean hasImportedSchemas(){

		return hasImportedSchemas;
	}

	public List<Schema> getImportedSchemas(){

		return importedSchemas;
	}

	public boolean hasTargetNamespace(){

		return hasTargetNamespace;
	}

	public String getTargetNamespace(){

		return targetNamespace;
	}

	protected boolean hasAnnotation(){

		return hasAnnotation;
	}

	protected Annotation getAnnotation(){

		return annotation;
	}

	protected String printQualifiers(){

		List<String> qualifiers = null;

		if( hasGlobalQualifier || hasSchemaQualifiers ) qualifiers = new ArrayList<String>();


		if( hasGlobalQualifier ) qualifiers.add( globalQualifier );

		for( int i = 0; hasSchemaQualifiers && i < schemaQualifiers.size(); ++i ) qualifiers.add( schemaQualifiers.get( i ) );


		String label = "";

		if( qualifiers != null ) label += " xmlns: " + qualifiers + "= '" + targetNamespace + "'";

		if( qualifiers == null && hasTargetNamespace ) label += " targetNamespace= '" + targetNamespace + "'" ;


		if( hasRemainingQualifierNamespaces )

			for ( Enumeration<String> e = ( (Hashtable<String, String>) remainingQualifierNamespaces ).keys(); e.hasMoreElements(); ){
				
				String qualifier = e.nextElement();

				String namespace = remainingQualifierNamespaces.get( qualifier );

				label += " xmlns:" + qualifier + "= '" + namespace + "'";
			}


		return label;
	}

	protected String getSchemaKind(){

		return schemaKind;
	}

	protected Map<String, String> getAttributes(){

		return tagAttributes;
	}

	protected double getSize(){

		return size;
	}

	protected int getMyOrderID(){

		return myOrderID;
	}

	private String printMyOrderID(){

		//return PrintMessages.get( PrintMessages.xmlPrint ) ? "(orderID= " + getMyOrderID() + ")" : "";
		return " (orderID= " + getMyOrderID() + ")";
	}

	private String printScope(){

		//return PrintMessages.get( PrintMessages.xmlPrint ) ? "(orderID= " + getMyOrderID() + ")" : "";
		return scope != null ? scope.getIntEndpoints() + " : " : "";
	}

	public List<?> getChildrenWithoutAnnotation(){

		return ListTransformations.merge( importedSchemas, globalComponents );
	}

	public List<?> getChildren(){

		return ListTransformations.merge( getChildrenWithoutAnnotation(), annotation );
	}

	public Interval1D getScope(){

		return scope;
	}

	public Object getParent(){

		return parent;
	}


	//Visualization.
	public DefaultMutableTreeNode visualize(){//This function is used by the 'Visualizer' class.

		DefaultMutableTreeNode node = new DefaultMutableTreeNode( printScope() + getTag() + printMyOrderID() );


		if( hasImportedSchemas ){
	
			DefaultMutableTreeNode importedSchemasNode = new DefaultMutableTreeNode( "Imported Schemas:" );
	
			node.add( importedSchemasNode );
	
			for( int i = 0; i < importedSchemas.size(); ++i ) importedSchemasNode.add( importedSchemas.get( i ).visualize() );
		}
	
		if( hasGlobalComponents ){
	
			DefaultMutableTreeNode componentsNode = new DefaultMutableTreeNode( globalComponents.size() + " components:" );
	
			node.add( componentsNode );

			for( int i = 0; i < globalComponents.size(); ++i ) componentsNode.add( globalComponents.get( i ).visualize() );
		}

		if( hasAnnotation ) node.add( annotation.visualize() );


		return node;
    }
}
