/**
 * @author Kostas Lekkas -- lekkas@vtrip.net
 */
package org.ow2.choreos.executable_service_discovery.plugin_manager;

/**
 * All plugins must extend the AbstractPlugin class, thus defining a new plugin 
 * type. The plugin manager can create multiple instances of the same plugin 
 * type (e.g. for N UDDI registries the plugin manager would have N instances
 * of the 'UDDI' plugin type)
 * 
 * Each plugin instance has the responsibility to deliver service-related 
 * information from the underlying service registry to the plugin manager. Based 
 * on how the plugin <-> service registry communication takes place, the plugins 
 * are categorized by their 'supported discovery mode' : active , passive , 
 * passive & active.
 * 
 * Plugins that operate in 'passive discovery mode' are passive receivers of 
 * messages from the underlying service registry. These messages indicate that 
 * a service has been added/removed/updated in the registry. As soon as a 
 * registry message is received, it is up to the plugin to inform 
 * the plugin manager accordingly. In the event of a service addition or update,
 * the plugin must also provide the respective service description in a unified
 * format. This 'unified format' is currently being defined.
 * 
 * On the other hand, plugins that operate in 'active discovery mode' must
 * explicitly request service descriptions from the underlying registry. This
 * mode is needed for registries that do not provide an eventing mechanism.
 * Plugins that support this mode implement the 'retrieveServices()' method, 
 * which is invoked by the Plugin Manager. The 'retrieveServices()' method call 
 * results in the (indirect and asynchronous) retrieval of all services that 
 * match a specific 'ServiceFilter', which is passed as a parameter. The exact 
 * format of the 'ServiceFilter' has yet to be formally defined and will 
 * probably take into account numerous parameters like service semantics, 
 * underlying transport protocol, request timeout, upper/lower limit to the 
 * number of services returned etc. 
 * 
 * Finally, a plugin could support both modes (e.g. DPWS/WS-Discovery)
 */
public abstract class AbstractPlugin {
	
    public static final char PASSIVE_DISCOVERY_MODE = 0x01;
    public static final char ACTIVE_DISCOVERY_MODE = 0x02;
    public static final char PASSIVE_ACTIVE_DISCOVERY_MODE = 0x03;
    
    public static final char STATUS_STARTING = 0x01;
    public static final char STATUS_STARTED = 0x02;
    public static final char STATUS_STOPPED = 0x03;
    public static final char STATUS_STOPPING = 0x04;
	
    /**
     * @return UID - fully qualified class name
     */
    public abstract String getUID();
    
    /**
     * @return short name of the specific plugin type (e.g 'DPWS')
     */
    public abstract String getShortName();
    
    /**
     * @return long name of the specific plugin type 
     * (e.g 'DPWS service discovery protocol')
     */
    public abstract String getLongName();
    
    /**
     * @return description of this plugin type.
     */
    public abstract String getDescription();
    
    /**
     * @return author name
     */
    public abstract String getAuthor();
    
    /**
     * @return supported discovery mode
     */
    public abstract char getSupportedDiscoveryMode();

    /**
     * @return the associated registry's URI.
     */
    public abstract String getRegistryURI();
    
    /**
     * @return current status of the plugin instance (running, stopped)
     */
    public abstract char getStatus();
    
    /**
     * This method results in the indirect and asynchronous retrieval
     * of all services matching the specified service filter.
     * It has no effect on plugins that only support passive discovery mode.
     * 
     * @param service_filter A service filter for the requested services.
     */
    public abstract Object retrieveServices(ServiceFilter service_filter);
    
    /*
     * Starts the plugin. 
     */
    public abstract void start();
    
    /*
     * Stops the plugin. 
     */
    public abstract void stop();
    
    /*
     * Restarts the plugin.
     */
    public abstract void restart();
    
}
